From 30f8a30be575aaa69ccad8c8eec26ac7737c2e78 Mon Sep 17 00:00:00 2001
From: Navid Rahimi <navidrahimi@microsoft.com>
Date: Mon, 22 Nov 2021 16:46:44 -0800
Subject: [PATCH] PR tree-optimization/101807

	* match.pd (bool0 < bool1) -> (!bool0 & bool1): New optimization.
	* match.pd  (bool0 <= bool1) -> (!bool0 | bool1): New optimization.
	* match.pd  (bool0 > bool1) -> (bool0 & !bool1): New optimization.
	* match.pd  (bool0 >= bool1) -> (bool0 | !bool1): New optimization.
	* gcc.dg/tree-ssa/pr101807.c: testcase for this optimization.
---
 gcc/match.pd                             | 33 ++++++++++++++++++++++++
 gcc/testsuite/gcc.dg/tree-ssa/pr101807.c | 33 ++++++++++++++++++++++++
 2 files changed, 66 insertions(+)
 create mode 100644 gcc/testsuite/gcc.dg/tree-ssa/pr101807.c

diff --git a/gcc/match.pd b/gcc/match.pd
index a319aefa808..50ddb6d9e8c 100644
--- a/gcc/match.pd
+++ b/gcc/match.pd
@@ -4449,6 +4449,39 @@ DEFINE_INT_AND_FLOAT_ROUND_FN (RINT)
  )
 )
 
+/* (bool0 < bool1) -> (!bool0 & bool1) */
+(simplify
+ (lt truth_valued_p@0 truth_valued_p@1)
+ (if (TREE_CODE (type) == BOOLEAN_TYPE
+      && types_match (type, TREE_TYPE (@0))
+      && types_match (TREE_TYPE (@0), TREE_TYPE (@1)))
+  (bit_and (bit_not @0) @1)))
+
+/* (bool0 <= bool1) -> (!bool0 | bool1) */
+(simplify
+ (le truth_valued_p@0 truth_valued_p@1)
+ (if (TREE_CODE (type) == BOOLEAN_TYPE
+      && types_match (type, TREE_TYPE (@0))
+      && types_match (TREE_TYPE (@0), TREE_TYPE (@1)))
+  (bit_ior (bit_not @0) @1)))
+
+/* (bool0 > bool1) -> (bool0 & !bool1) */
+(simplify
+ (gt truth_valued_p@0 truth_valued_p@1)
+ (if (TREE_CODE (type) == BOOLEAN_TYPE
+      && types_match (type, TREE_TYPE (@0))
+      && types_match (TREE_TYPE (@0), TREE_TYPE (@1)))
+  (bit_and @0 (bit_not @1))))
+
+/* (bool0 >= bool1) -> (bool0 | !bool1) */
+(simplify
+ (ge truth_valued_p@0 truth_valued_p@1)
+ (if (TREE_CODE (type) == BOOLEAN_TYPE
+      && types_match (type, TREE_TYPE (@0))
+      && types_match (TREE_TYPE (@0), TREE_TYPE (@1)))
+  (bit_ior @0 (bit_not @1))))
+
+
 /* -(type)!A -> (type)A - 1.  */
 (simplify
  (negate (convert?:s (logical_inverted_value:s @0)))
diff --git a/gcc/testsuite/gcc.dg/tree-ssa/pr101807.c b/gcc/testsuite/gcc.dg/tree-ssa/pr101807.c
new file mode 100644
index 00000000000..29012ca1fb1
--- /dev/null
+++ b/gcc/testsuite/gcc.dg/tree-ssa/pr101807.c
@@ -0,0 +1,33 @@
+/* PR tree-optimization/101807 */
+/* { dg-do compile } */
+/* { dg-options "-O -fdump-tree-optimized" } */
+
+#include <stdbool.h>
+
+bool g(bool a, bool b)
+{
+    return (a < b);
+}
+
+bool h(bool a, bool b)
+{
+  return (a <= b);
+}
+
+bool i(bool a, bool b)
+{
+  return (a > b);
+}
+
+bool j(bool a, bool b)
+{
+  return (a >= b);
+}
+
+
+/* Make sure we have removed < and <= from "a < b" and "a <= b". */
+/* { dg-final { scan-tree-dump-not " < " "optimized" } } */
+/* { dg-final { scan-tree-dump-not " <= " "optimized" } } */
+/* Make sure we have removed > and >= from "a > b" and "a >= b". */
+/* { dg-final { scan-tree-dump-not " > " "optimized" } } */
+/* { dg-final { scan-tree-dump-not " >= " "optimized" } } */
\ No newline at end of file
-- 
2.25.1

